import 'dart:async';

import 'package:sudoku/provider/data.dart';

class BoardAnimate {
  // Start game animation: blinking tiles
  static List<List<List<bool>>> createStartGameAnimationPatterns(Data myProvider) {
    List<List<List<bool>>> patterns = [];

    int patternsCount = 3;
    int boardSideLength = myProvider.blockSizeHorizontal * myProvider.blockSizeVertical;

    for (int patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      List<List<bool>> pattern = [];
      for (int row = 0; row < boardSideLength; row++) {
        List<bool> patternRow = [];
        for (int col = 0; col < boardSideLength; col++) {
          patternRow.add(((patternIndex + row + col) % 2 == 0));
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  // Win game animation: fill board with colored rows, from bottom to top
  static List<List<List<bool>>> createWinGameAnimationPatterns(Data myProvider) {
    List<List<List<bool>>> patterns = [];

    int boardSideLength = myProvider.blockSizeHorizontal * myProvider.blockSizeVertical;
    int patternsCount = boardSideLength + 6;

    for (int patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      List<List<bool>> pattern = [];
      for (int row = 0; row < boardSideLength; row++) {
        List<bool> patternRow = [];
        for (int col = 0; col < boardSideLength; col++) {
          patternRow.add(row > (patternIndex - 4));
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  // Default multi-purpose animation: sliding stripes, from top left to right bottom
  static List<List<List<bool>>> createDefaultAnimationPatterns(Data myProvider) {
    List<List<List<bool>>> patterns = [];

    int boardSideLength = myProvider.blockSizeHorizontal * myProvider.blockSizeVertical;
    int patternsCount = boardSideLength;

    for (int patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      List<List<bool>> pattern = [];
      for (int row = 0; row < boardSideLength; row++) {
        List<bool> patternRow = [];
        for (int col = 0; col < boardSideLength; col++) {
          patternRow.add(((patternIndex + row + col) % 4 == 0));
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  static void startAnimation(Data myProvider, String animationType) {
    List<List<List<bool>>> patterns = [];

    switch (animationType) {
      case 'start':
        patterns = createStartGameAnimationPatterns(myProvider);
        break;
      case 'win':
        patterns = createWinGameAnimationPatterns(myProvider);
        break;
      default:
        patterns = createDefaultAnimationPatterns(myProvider);
    }

    int patternIndex = patterns.length;

    myProvider.updateAnimationInProgress(true);

    const interval = Duration(milliseconds: 200);
    Timer.periodic(
      interval,
      (Timer timer) {
        if (patternIndex == 0) {
          timer.cancel();
          myProvider.resetAnimatedBackground();
          myProvider.updateAnimationInProgress(false);
        } else {
          patternIndex--;
          myProvider.setAnimatedBackground(patterns[patternIndex]);
        }
      },
    );
  }
}
